@@ -20,8 +20,8 @@
 # Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
 # Boston, MA 02110-1301, USA.
 
+from gettext import gettext as _
 import gobject
-gobject.threads_init()
 import gst
 
 from xml.etree.ElementTree import Element, SubElement, tostring, parse
@@ -84,6 +84,9 @@ class ElementTreeFormatter(Formatter):
         self.timelinenode = None
         # An Element representing the <export-settings> element.
         self._settingsnode = None
+        # An Element representing the <metadata> element.
+        self._metadatanode = None
+
         # A list of SourceFactory objects.
         self._sources = None
         self._context = ElementTreeFormatterContext()
@@ -102,6 +105,8 @@ class ElementTreeFormatter(Formatter):
             yield name, value
 
     def _parsePropertyValue(self, value):
+        if value == "":
+            return value
         # We treat the GEnum values differently because they are actually ints.
         if "(GEnum)" in value:
             return int(value.split("(GEnum)")[1])
@@ -110,8 +115,12 @@ class ElementTreeFormatter(Formatter):
         # and gst.Caps() fails to parse it, because it has "L" at the end.
         if "(guint64)" in value:
             value = value.rstrip('lL')
-        # TODO: Use what gst.Caps() uses to parse a property value.
-        caps = gst.Caps("structure1, property1=%s;" % value)
+        try:
+            # TODO: Use what gst.Caps() uses to parse a property value.
+            caps = gst.Caps("structure1, property1=%s;" % value)
+        except TypeError, exception:
+            # Report the value which caused the exception.
+            raise Exception(exception, value)
         structure = caps[0]
         return structure["property1"]
 
@@ -289,8 +298,7 @@ class ElementTreeFormatter(Formatter):
         """Get the specified Element as a deserialized dict."""
         values_dict = {}
         for name, value_string in element.attrib.iteritems():
-            if value_string:
-                values_dict[name] = self._parsePropertyValue(value_string)
+            values_dict[name] = self._parsePropertyValue(value_string)
         return values_dict
 
     def _saveProjectSettings(self, settings):
@@ -321,6 +329,13 @@ class ElementTreeFormatter(Formatter):
             self._serializeDict(ss, settings.acodecsettings)
         return element
 
+    def _saveProjectMetadata(self, project):
+        element = Element('metadata')
+        element.attrib["author"] = project.author
+        element.attrib["name"] = project.name
+        element.attrib["year"] = project.year
+        return element
+
     def _loadProjectSettings(self, element):
         self.debug("element:%r", element)
         settings = ExportSettings()
@@ -351,6 +366,11 @@ class ElementTreeFormatter(Formatter):
 
         return settings
 
+    def _loadProjectMetadata(self, element, project):
+        project.name = element.attrib["name"]
+        project.author = element.attrib["author"]
+        project.year = element.attrib["year"]
+
     def _saveTrackObject(self, track_object):
         element = Element("track-object")
         element.attrib["id"] = self._new_element_id()
@@ -424,17 +444,15 @@ class ElementTreeFormatter(Formatter):
         try:
             factory = self.avalaible_effects.getFactoryFromName(factory_name)
         except KeyError:
-            #Find a way to figure out how we could install the missing effect...
-            raise FormatterError(_("The project contains effects that are not\
-                                   avalaibe on the system. It can't be loaded"))
+            # TODO: Find a way to install the missing effect.
+            raise FormatterError(_("The project contains effects which are not "
+                                   "available on the system."))
 
         input_stream = factory.getInputStreams()
         if not input_stream:
             raise FormatterError("cant find effect factory input stream")
         input_stream = input_stream[0]
-
         track_object = klass(factory, input_stream)
-
         track.addTrackObject(track_object)
 
         for name, value_string in self._filterElementProperties(element):
@@ -699,6 +717,9 @@ class ElementTreeFormatter(Formatter):
         if project.settings:
             root.append(self._saveProjectSettings(project.settings))
 
+        # metadata
+        root.append(self._saveProjectMetadata(project))
+
         # sources
         root.append(self._saveFactories(project.sources.getSources()))
 
@@ -717,22 +738,26 @@ class ElementTreeFormatter(Formatter):
 
         return True
 
-    def _loadProject(self, location, project):
-        self.debug("location:%s, project:%r", location, project)
+    def _loadProject(self, project_uri, project):
+        self.debug("project_uri:%s, project:%r", project_uri, project)
         # open the given location
-        self._context.rootelement = parse(location.split('://', 1)[1])
+        self._context.rootelement = parse(project_uri.split('://', 1)[1])
         self.factoriesnode = self._context.rootelement.find("factories")
         self.timelinenode = self._context.rootelement.find("timeline")
         self._settingsnode = self._context.rootelement.find("export-settings")
-        if project and self._settingsnode != None:
+        self._metadatanode = self._context.rootelement.find("metadata")
+
+        if self._settingsnode != None:
             project.setSettings(self._loadProjectSettings(self._settingsnode))
 
+        if self._metadatanode != None:
+            self._loadProjectMetadata(self._metadatanode, project)
         # rediscover the factories
         closure = {"rediscovered": 0}
         try:
             sources = self._getSources()
         except FormatterError, e:
-            self.emit("new-project-failed", location, e)
+            self.emit("new-project-failed", project_uri, e)
             return
 
         uris = [source.uri for source in sources]
@@ -741,7 +766,7 @@ class ElementTreeFormatter(Formatter):
         discoverer.connect("discovery-done", self._discovererDiscoveryDoneCb,
                 project, sources, uris, closure)
         discoverer.connect("discovery-error", self._discovererDiscoveryErrorCb,
-                project, sources, uris, closure)
+                project, sources, uris, closure, project_uri)
 
         if not sources:
             self._finishLoadingProject(project)
@@ -828,19 +853,28 @@ class ElementTreeFormatter(Formatter):
         discoverer.addUri(next.uri)
 
     def _discovererDiscoveryErrorCb(self, discoverer, uri, error, detail,
-            project, sources, uris, closure):
+            project, sources, uris, closure, project_uri):
         if uri not in uris:
             # someone else is using discoverer, this signal isn't for us
             return
 
-        self.emit("new-project-failed", uri,
-                FormatterError("%s: %s" % (error, detail)))
+        message = _("Failed loading %(uri)s.") % {"uri": uri}
+        message += "\n\n%s" % error
+        if detail:
+            message += "\n\n%s" % detail
+        formatter_error = FormatterError(message)
+        self.emit("new-project-failed", project_uri, formatter_error)
 
     def newProject(self):
         project = Formatter.newProject(self)
         # add the settings
         if self._settingsnode != None:
             project.setSettings(self._loadProjectSettings(self._settingsnode))
+
+        # add metadata
+        if self._metadatanode != None:
+            self._loadProjectMetadata(self._metadatanode, project)
+
         return project
 
     def _getSources(self):
