@@ -1,4 +1,3 @@
-#!/usr/bin/python
 # PiTiVi , Non-linear video editor
 #
 #       previewer.py
@@ -17,8 +16,8 @@
 #
 # You should have received a copy of the GNU Lesser General Public
 # License along with this program; if not, write to the
-# Free Software Foundation, Inc., 59 Temple Place - Suite 330,
-# Boston, MA 02111-1307, USA.
+# Free Software Foundation, Inc., 51 Franklin St, Fifth Floor,
+# Boston, MA 02110-1301, USA.
 
 """
 Utility tools and classes for easy generation of previews
@@ -51,11 +50,35 @@ GlobalSettings.addConfigOption("thumbnailSpacingHint",
     default=2,
     notify=True)
 
+GlobalSettings.addConfigOption("thumbnailPeriod",
+    section="thumbnailing",
+    key="thumbnail-period",
+    default=gst.SECOND,
+    notify=True)
+
 PreferencesDialog.addNumericPreference("thumbnailSpacingHint",
     section=_("Appearance"),
-    label=_("Thumbnail Gap (pixels)"),
+    label=_("Thumbnail gap"),
     lower=0,
-    description=_("The gap between thumbnails"))
+    description=_("The spacing between thumbnails, in pixels"))
+
+PreferencesDialog.addChoicePreference("thumbnailPeriod",
+    section=_("Performance"),
+    label=_("Thumbnail every"),
+    choices=(
+        # Note that we cannot use "%s second" or ngettext, because fractions
+        # are not supported by ngettext and their plurality is ambiguous
+        # in many languages.
+        # See http://www.gnu.org/software/hello/manual/gettext/Plural-forms.html
+        (_("1/100 second"), gst.SECOND / 100),
+        (_("1/10 second"), gst.SECOND / 10),
+        (_("1/4 second"), gst.SECOND / 4),
+        (_("1/2 second"), gst.SECOND / 2),
+        (_("1 second"), gst.SECOND),
+        (_("5 seconds"), 5 * gst.SECOND),
+        (_("10 seconds"), 10 * gst.SECOND),
+        (_("minute"), 60 * gst.SECOND)),
+    description=_("The interval, in seconds, between thumbnails."))
 
 # this default works out to a maximum of ~ 1.78 MiB per factory, assuming:
 # 4:3 aspect ratio
@@ -71,29 +94,29 @@ GlobalSettings.addConfigOption("thumbnailCacheSize",
 GlobalSettings.addConfigOption("thumbnailMaxRequests",
     section="thumbnailing",
     key="max-requests",
-    default = 10)
+    default=10)
 
 GlobalSettings.addConfigOption('showThumbnails',
-    section = 'user-interface',
-    key = 'show-thumbnails',
-    default = True,
-    notify = True)
+    section='user-interface',
+    key='show-thumbnails',
+    default=True,
+    notify=True)
 
 PreferencesDialog.addTogglePreference('showThumbnails',
-    section = _("Appearance"),
-    label = _("Show Thumbnails (Video)"),
-    description = _("Show Thumbnails on Video Clips"))
+    section=_("Performance"),
+    label=_("Enable video thumbnails"),
+    description=_("Show thumbnails on video clips"))
 
 GlobalSettings.addConfigOption('showWaveforms',
-    section = 'user-interface',
-    key = 'show-waveforms',
-    default = True,
-    notify = True)
+    section='user-interface',
+    key='show-waveforms',
+    default=True,
+    notify=True)
 
 PreferencesDialog.addTogglePreference('showWaveforms',
-    section = _("Appearance"),
-    label = _("Show Waveforms (Audio)"),
-    description = _("Show Waveforms on Audio Clips"))
+    section=_("Performance"),
+    label=_("Enable audio waveforms"),
+    description=_("Show waveforms on audio clips"))
 
 # Previewer                      -- abstract base class with public interface for UI
 # |_DefaultPreviewer             -- draws a default thumbnail for UI
@@ -107,6 +130,7 @@ PreferencesDialog.addTogglePreference('showWaveforms',
 
 previewers = {}
 
+
 def get_preview_for_object(instance, trackobject):
     factory = trackobject.factory
     stream_ = trackobject.stream
@@ -128,10 +152,11 @@ def get_preview_for_object(instance, trackobject):
             previewers[key] = DefaultPreviewer(instance, factory, stream_)
     return previewers[key]
 
+
 class Previewer(Signallable, Loggable):
 
     __signals__ = {
-        "update" : ("segment",),
+        "update": ("segment",),
     }
 
     # TODO: parameterize height, instead of assuming self.theight pixels.
@@ -158,14 +183,15 @@ class Previewer(Signallable, Loggable):
     def _connectSettings(self, settings):
         self._settings = settings
 
+
 class DefaultPreviewer(Previewer):
 
     def render_cairo(self, cr, bounds, element, hscroll_pos, y1):
         # TODO: draw a single thumbnail
         pass
 
-class RandomAccessPreviewer(Previewer):
 
+class RandomAccessPreviewer(Previewer):
     """ Handles loading, caching, and drawing preview data for segments of
     random-access streams.  There is one Previewer per stream per
     ObjectFactory.  Preview data is read from an instance of an
@@ -223,7 +249,7 @@ class RandomAccessPreviewer(Previewer):
 
         # tdur = duration in ns of thumbnail
         # sof  = start of file in pixel coordinates
-        x1 = bounds.x1;
+        x1 = bounds.x1
         sof = Zoomable.nsToPixel(element.start - element.in_point) +\
             hscroll_pos
 
@@ -338,6 +364,7 @@ class RandomAccessPreviewer(Previewer):
         self.spacing = settings.thumbnailSpacingHint
         self.emit("update", None)
 
+
 class RandomAccessVideoPreviewer(RandomAccessPreviewer):
 
     @property
@@ -367,18 +394,18 @@ class RandomAccessVideoPreviewer(RandomAccessPreviewer):
             (self.theight, self.twidth + 2))
         filter_ = utils.filter_(caps)
         self.videopipeline = utils.pipeline({
-            sbin : csp,
-            csp : scale,
-            scale : filter_,
-            filter_ : sink,
-            sink : None
+            sbin: csp,
+            csp: scale,
+            scale: filter_,
+            filter_: sink,
+            sink: None
         })
         sink.connect('thumbnail', self._thumbnailCb)
         self.videopipeline.set_state(gst.STATE_PAUSED)
 
     def _segment_for_time(self, time):
         # quantize thumbnail timestamps to maximum granularity
-        return time - (time % self.tstep)
+        return utils.quantize(time, self.tperiod)
 
     def _thumbnailCb(self, unused_thsink, pixbuf, timestamp):
         gobject.idle_add(self._finishThumbnail, pixbuf, timestamp)
@@ -393,17 +420,25 @@ class RandomAccessVideoPreviewer(RandomAccessPreviewer):
     def _connectSettings(self, settings):
         RandomAccessPreviewer._connectSettings(self, settings)
         settings.connect("showThumbnailsChanged", self._showThumbsChanged)
+        settings.connect("thumbnailPeriodChanged",
+            self._thumbnailPeriodChanged)
         self._view = settings.showThumbnails
+        self.tperiod = settings.thumbnailPeriod
 
     def _showThumbsChanged(self, settings):
         self._view = settings.showThumbnails
         self.emit("update", None)
 
-class StillImagePreviewer(RandomAccessVideoPreviewer):
+    def _thumbnailPeriodChanged(self, settings):
+        self.tperiod = settings.thumbnailPeriod
+        self.emit("update", None)
 
+
+class StillImagePreviewer(RandomAccessVideoPreviewer):
     def _thumbForTime(self, cr, time, x, y):
         return RandomAccessVideoPreviewer._thumbForTime(self, cr, 0L, x, y)
 
+
 class RandomAccessAudioPreviewer(RandomAccessPreviewer):
 
     def __init__(self, instance, factory, stream_):
@@ -421,9 +456,9 @@ class RandomAccessAudioPreviewer(RandomAccessPreviewer):
         self.audioSink = ArraySink()
         conv = gst.element_factory_make("audioconvert")
         self.audioPipeline = utils.pipeline({
-            sbin : conv,
-            conv : self.audioSink,
-            self.audioSink : None})
+            sbin: conv,
+            conv: self.audioSink,
+            self.audioSink: None})
         bus = self.audioPipeline.get_bus()
         bus.add_signal_watch()
         bus.connect("message::segment-done", self._busMessageSegmentDoneCb)
@@ -476,7 +511,7 @@ class RandomAccessAudioPreviewer(RandomAccessPreviewer):
                width, self.theight)
             cr = cairo.Context(scaled)
             matrix = cairo.Matrix()
-            matrix.scale(self.base_width/width, 1.0)
+            matrix.scale(self.base_width / width, 1.0)
             cr.set_source_surface(surface)
             cr.get_source().set_matrix(matrix)
             cr.rectangle(0, 0, width, self.theight)
@@ -560,4 +595,3 @@ class RandomAccessAudioPreviewer(RandomAccessPreviewer):
     def _showWaveformsChanged(self, settings):
         self._view = settings.showWaveforms
         self.emit("update", None)
-
